#pragma once

#ifndef _MathFunctionOnGrid_H
#define _MathFunctionOnGrid_H

#include <string>
#include <vector>

#include "MathCommon.h"

class MathFunctionOnGrid
{
public:

	// Create a function on grid through a given funtional form
	MathFunctionOnGrid(const NDData& xmin, const NDData& xmax, const NDInt& numberGrid, void* pSomething,
		void (* GetFunctionValues)(const NDDataSet& xDataSet, OneDDataSet& yDataSet, void* pSomething));

	// Create a function on grid from a set of data-- N-Dim case
	MathFunctionOnGrid(const NDDataSet& x, const OneDDataSet& y): 
		m_xDataSet(x),m_yDataSet(y){};

	// Create a function on grid from a set of data-- 1-Dim case
	MathFunctionOnGrid(const OneDDataSet& x, const OneDDataSet& y); 

	~MathFunctionOnGrid(void);

	// Copy out the grids and values-- N-Dim case
	void GetValues(NDDataSet& x, OneDDataSet& y) const;

	// Copy out the grids and values-- 1-Dim case
	void GetValues(OneDDataSet& x, OneDDataSet& y) const;

public:

	static MathFunctionOnGrid Integrate(const MathFunctionOnGrid& derivative);
	static double DefiniteIntegral(const MathFunctionOnGrid& function);
	static double DefiniteIntegral(const OneDDataSet& x, const OneDDataSet& y);
	static double LogDefiniteIntegralOnExp(const OneDDataSet& x, const OneDDataSet& y);
	static void Gaussian(const double& mean, const double& variance, const OneDDataSet& xDataSet, OneDDataSet& yDataSet, bool normalized=true);
	static void Quadratic(const double& a, const double& x0, const OneDDataSet& xDataSet, OneDDataSet& yDataSet);

	static void CubicSpline(const OneDDataSet& node, const OneDDataSet& nodeValue, const OneDDataSet& xDataSet, OneDDataSet& f, bool periodic=false);
	static void CubicSpline(const NDDataSet& node, const OneDDataSet& nodeValue, const NDDataSet& xDataSet, OneDDataSet& f, bool periodic=false);

	static void CubicSpline(const NDDataSet& node, const OneDDataSet& nodeValue, const NDDataSet& xDataSet, OneDDataSet& f, NDDataSet& gradient, bool periodic=false);
	// the x-derivative in gradient[0]
	// the x-2nd-derivative in gradient[1]
	// the y-derivative in gradient[2]
	// the y-2nd-derivative in gradient[3]
	// the xy cross-2nd-derivative in gradient[4]

	static void CubicSplineGetZeroGradient(const OneDDataSet& splineTable, const NDData& delta, NDDataSet& zeroGradient, NDDataSet& stationary, const bool maxIncluded=false);

	static void ResampleCubicSpline(const OneDDataSet& inNode, const OneDDataSet& inNodeValue, const OneDDataSet& outNode, OneDDataSet& outNodeValue);
	static void ResampleCubicSpline(const NDDataSet& inNode, const OneDDataSet& inNodeValue, const NDDataSet& outNode, OneDDataSet& outNodeValue);

	static void Rational(const OneDDataSet& node, const OneDDataSet& nodeValue, const OneDDataSet& xDataSet, OneDDataSet& yDataSet, int order=5, bool periodic=false);

protected:

	MathFunctionOnGrid(void);

private:

	void PopulateGrid(int currentDim, const NDData& xmin, const NDData& xmax, const NDInt& numberGrid);

	NDDataSet m_xDataSet;
	OneDDataSet m_yDataSet;

};

#endif //
