//  MatrixTools.h
//
//////////////////////////////////////////////////////////////////////

#pragma once

#ifndef _MatrixTools_H
#define _MatrixTools_H

#include <math.h>
#include <string>
#include <vector>
#include <iostream>

#include <tnt.h>
using namespace TNT;
using namespace JAMA;

template <class T>
bool operator == (const Array1D<T> &A, const Array1D<T> &B)
{

	int n = A.dim1();
	int m = B.dim1();
	
	if (n!=m) return false;
	
	bool result=true;
	for (int i=0; i<n; ++i){
		if (A[i]!=B[i]){
			result=false;
			break;
		}
	}
	
	return result;

}

template <class T>
bool operator != (const Array1D<T> &A, const Array1D<T> &B)
{
	return ~(A==B);
}

template <class T>
T Sum(const Array2D<T> &A)
{

	int n = A.dim1();
	int m = A.dim2();
	
	T sum=0;
	
	for (int i=0; i<n; ++i){
		for (int j=0; j<m; ++j){
			sum+=A[i][j];
		}
	}
	
	return sum;

}

template <class T>
T Abs(const Array2D<T> &A)
{

	Array2D<T> A2=A*A;
	return Sum(A2);

}

template <class T>
T Distance(const Array2D<T> &A, const Array2D<T> &B)
{

	if (A.dim1()!=B.dim1() || A.dim2()!=B.dim2() ) return -1;
	Array2D<T> C=A-B;
	return Abs(C);

}
#endif //
