#pragma once

#ifndef _Maximizer_H
#define _Maximizer_H

#include <dlib/array.h>
#include "alglib/src/ap.h"

#include "MathCommon.h"

class Maximizer
{
public:

	Maximizer(const unsigned nIter=1000, const double delta=1e-6, const bool derivative=false):
		maxIter(nIter), tol(delta), hasDerivative(derivative)
	{
		normalFactor=1.0;
		converged=false;
	};
	~Maximizer(){};

	void Maximize(OneDDataSet& parameters);

	virtual double FunctionValue(const OneDDataSet& arg)=0; 
	virtual OneDDataSet Derivative(const OneDDataSet& arg) const =0;

protected:

	enum ExtLib { Alg, dLib };
	ExtLib lib;	

	int maxIter;
	double tol;
	bool hasDerivative;

	bool converged;
	double normalFactor;

private:

	void Maximize_Alg(alglib::real_1d_array& parameters);
	void Maximize_dLib(dlib_ParameterSet& parametersInVector);

	double FunctionValue_Alg(const alglib::real_1d_array& parameters);
	double FunctionValue_dLib(const dlib_ParameterSet& parameters);

	alglib::real_1d_array Derivative_Alg(const alglib::real_1d_array& parameters);
	dlib_ParameterSet Derivative_dLib(const dlib_ParameterSet& parameters);

	Maximizer(const Maximizer&);
};

#endif // _Maximizer_H
