#pragma once

#ifndef _ModelFunction_H
#define _ModelFunction_H

#include <math.h>
#include <string>
#include <vector>
#include <iostream>

#include "MathCommon.h"

class ModelFunction
{

public:

	enum functionType {
		typeOneD_ExpAndSixOrder,
		typeOneD_ExpCos,
		typeOneD_FT,
		typeOneD_Test,
		typeOneD_Quadratic,
		typeOneD_Order6,
		typeOneD_Order4,
		typeOneD_CubicSpline,
		typeOneD_CyclicCubicSpline,
		typeOneD_Rational,
		typeOneD_CyclicRational,
		typeOneD_BEDAM,
	};

	ModelFunction(const ModelFunction::functionType type=typeOneD_Order4, const int numberParameter=99);

	void SetFeature(const NDDataSet& feature)
		{ m_feature=feature; };
	const NDDataSet& GetFeature() const{ return m_feature; };

	const OneDDataSet& GetInitialParameter() const{ return m_initialParameter; };
	void SetInitialParameter(const OneDDataSet& parameter){ m_initialParameter=parameter; };
	
	void (* GetFunctionPointer()) 
		(const NDDataSet& feature,
		const OneDDataSet& parameter,
		const NDDataSet& x,
		OneDDataSet& y) 
		{ return m_pFunction; };
	
private:

	void (*m_pFunction)			
		(const NDDataSet& feature,
		const OneDDataSet& parameter,
		const NDDataSet& x,
		OneDDataSet& f); 

	OneDDataSet m_initialParameter;
	NDDataSet m_feature;
	int m_numberParameter;

private:

	ModelFunction(const ModelFunction&);

private:

	static void ExpAndSixOrder
		(const NDDataSet& feature,
		const OneDDataSet& parameter,
		const NDDataSet& x,
		OneDDataSet& f); 
	
	static void ExpCos
		(const NDDataSet& feature,
		const OneDDataSet& parameter,
		const NDDataSet& x,
		OneDDataSet& f); 

	static void FT
		(const NDDataSet& feature,
		const OneDDataSet& parameter,
		const NDDataSet& x,
		OneDDataSet& f); 

	static void Test
		(const NDDataSet& feature,
		const OneDDataSet& parameter,
		const NDDataSet& x,
		OneDDataSet& f); 

	static void Quadratic
		(const NDDataSet& feature,
		const OneDDataSet& parameter,
		const NDDataSet& x,
		OneDDataSet& f); 

	static void Order6
		(const NDDataSet& feature,
		const OneDDataSet& parameter,
		const NDDataSet& x,
		OneDDataSet& f); 

	static void Order4
		(const NDDataSet& feature,
		const OneDDataSet& parameter,
		const NDDataSet& x,
		OneDDataSet& f); 

	static void CubicSpline
		(const NDDataSet& feature,
		const OneDDataSet& parameter,
		const NDDataSet& x,
		OneDDataSet& f); 

	static void CyclicCubicSpline
		(const NDDataSet& feature,
		const OneDDataSet& parameter,
		const NDDataSet& x,
		OneDDataSet& f); 

	static void Rational
		(const NDDataSet& feature,
		const OneDDataSet& parameter,
		const NDDataSet& x,
		OneDDataSet& f); 

	static void CyclicRational
		(const NDDataSet& feature,
		const OneDDataSet& parameter,
		const NDDataSet& x,
		OneDDataSet& f); 

	static void BEDAM
		(const NDDataSet& feature,
		const OneDDataSet& parameter,
		const NDDataSet& x,
		OneDDataSet& f); 

};


#endif // _ModelFunction_H
