//  RotationTools.h
//
//////////////////////////////////////////////////////////////////////

#pragma once

#ifndef _RotationTools_H
#define _RotationTools_H

#include <math.h>
#include <string>
#include <vector>
#include <iostream>

#include <tnt.h>
using namespace TNT;
using namespace JAMA;

template <class T>
Array2D<T> UnitaryRotation(const T& theta)
{

	Array2D<T> C(2,2);
	C[0][0]=cos(theta);
	C[1][1]=cos(theta);
	C[1][0]=sin(theta);
	C[0][1]=-sin(theta);
	
	return C;
}

template <class T>
T Positivity(const Array2D<T>& A)
{

	T result=0.0;

	for (int i=0; i<A.dim1(); ++i){
		for (int j=0; j<A.dim2(); ++j){
			T tt=fabs(A[i][j])-A[i][j];
			result+=tt*tt;
		}
	}
	
	return result;
}


template <class T>
Array2D<T> OptimizedRotation(const Array2D<T> &A)
{
    if (A.dim1() != 2)
        return Array2D<T>();

	T t1=-5.14159;
	T t2=0;
	T t3=5.14159;

	for (int i=0; i<20; ++i) {

		Array2D<T> r1=UnitaryRotation(t1);
		Array2D<T> r2=UnitaryRotation(t2);
		Array2D<T> r3=UnitaryRotation(t3);

		T result1=Positivity(matmult(r1,A));
		T result2=Positivity(matmult(r2,A));
		T result3=Positivity(matmult(r3,A));

		//std::cout<<t1<<"\t"<<t2<<"\t"<<t3<<"\t";
		//std::cout<<result1<<"\t"<<result2<<"\t"<<result3<<std::endl;

		if (result2<result1){
			t1=(t1+t2)/2.0;
		} else {
			t1=t1-(t2-t1)/2.0;
			t3=t2;
		}
		if (result2<result3){
			t3=(t3+t2)/2.0;
		} else {
			t3=t3-(t2-t3)/2.0;
			t1=t2;
		}
		t2=(t1+t3)/2.0;
	}

	Array2D<T> result=matmult(UnitaryRotation(t2),A);
	std::cout<<"Result:"<<Positivity(result)<<std::endl;
	return result;
}

#endif //
