// StatTools.h: interface for the StatTools class.
//
//
//////////////////////////////////////////////////////////////////////
#pragma once

#ifndef _StatTools_H
#define _StatTools_H

#include <math.h>
#include <string>
#include <vector>

template <class Real> class NormalDistribution
{

public:

	NormalDistribution(std::vector<Real>& data):
		m_data(data), m_mean(0), m_SD(0), m_max(-1.0e99), m_min(1.0e99){

		m_size=m_data.size();

		if (m_size>0){

			Real sum=0.0;
			for (int i=0; i<m_size; ++i){
				if (m_data[i]>m_max) m_max=m_data[i];
				if (m_data[i]<m_min) m_min=m_data[i];
				sum+=m_data[i];
			}
			m_mean=sum/(1.0*m_size);

			Real sdSum=0.0;	
			for (int j=0; j<m_size; ++j){
				Real tt=(m_data[j]-m_mean);
				sdSum+=tt*tt;
			}
			m_var=sdSum/(1.0*(m_size-1));
			m_SD=sqrt(m_var);
			//if (fabs(m_SD)<1e-10) m_SD=0.0;

		}

	}


	Real GetMean() const{ return m_mean; };
	Real GetStandardDeviation() const{ return m_SD; };
	Real GetVariance() const{ return m_var; };

	Real GetMax() const{ return m_max; };
	Real GetMin() const{ return m_min; };
	Real GetSize() const{ return m_size; };

private:

	std::vector<Real>& m_data;

	int m_size;
	Real m_mean, m_var, m_SD, m_max, m_min;

};

#endif // // !defined(AFX_STATTOOLS_H__0ACA4320_54AD_4465_BD22_E9FCD49F6769__INCLUDED_)
