#include "../include/MiscHelper.h"

void MiscHelper::CalculateLogZ(
	const unsigned dim,
	const OneDDataSet& splineTable,
	const std::vector<QPoint>& quadraturePointList,
	const unsigned numberWindow,
	std::vector<double>& logZ,
	double& xAve,
	const double sps){

	static OneDDataSet splineTableSaved;

	unsigned nx = 4, ny = dim*dim, shift = 2 * dim, inc = 4 * dim*dim + shift;
	unsigned numberQuadPoints = quadraturePointList.size();

	const double* p0 = splineTable.data();
	const QPoint* pq = quadraturePointList.data();

	static OneDDataSet pointContribution(0, 0.0);
	static OneDDataSet pointContribution_X(0, 0.0);
	static OneDDataSet pointContribution_X0(0, 0.0);
	static OneDDataSet Zsaved(0, 0.0);
	static OneDDataSet sum_x(0, 0.0);

	static OneDDataSet logw(0, 0.0);
	static OneDDataSet tt(0, 0.0);

	static std::vector<unsigned> tblIndex(0, 0);
	static std::vector<unsigned> winIndex(0, 0);
	static std::vector<unsigned> cc(0, 0);

	bool needUpdate = false;
	if (pointContribution.size() != numberQuadPoints) {

		Zsaved.assign(numberWindow, 0.0);
		sum_x.assign(numberWindow, 0.0);

		pointContribution.assign(numberQuadPoints, 0.0);
		pointContribution_X.assign(numberQuadPoints, 0.0);
		pointContribution_X0.assign(numberQuadPoints, 0.0);
		logw.assign(numberQuadPoints, 0.0);
		tt.assign(numberQuadPoints, 0.0);
		tblIndex.assign(numberQuadPoints, 0);
		winIndex.assign(numberQuadPoints, 0);
		cc.assign(numberQuadPoints, 0);
		for (unsigned iq = 0; iq < numberQuadPoints; iq++){
			tblIndex[iq] = quadraturePointList[iq].tblIndex;
			winIndex[iq] = quadraturePointList[iq].windowIndex;
			pointContribution_X0[iq] = quadraturePointList[iq].x[0];
			logw[iq] = quadraturePointList[iq].logw;
		}
		needUpdate = true;
	}

	logZ = Zsaved;

	double* plogz = logZ.data();
	double* pZsaved = Zsaved.data();
	double* psum_x = sum_x.data();

	double* pc = pointContribution.data();
	double* pc_X = pointContribution_X.data();
	double* pc_X0 = pointContribution_X0.data();
	double* pLogw = logw.data();
	double* ptt = tt.data();

	unsigned *pTbl = tblIndex.data();
	unsigned *pWin = winIndex.data();
	unsigned *pcc = cc.data();

	std::vector<int> changed(splineTable.size() / inc, true);
	int* pChanged = changed.data();

	if (sps > 1e-50 && !needUpdate){
		int counter = 0;
		if (splineTableSaved.size() == splineTable.size()){
			const double* p0 = splineTable.data();
			const double* p1 = splineTableSaved.data();

			unsigned tableSize = changed.size();
			for (unsigned i = 0; i < tableSize; i++){
				changed[i] = false;
				unsigned ii = i*inc;
				for (unsigned j = 0; j < inc; j++){
					unsigned k = j + ii;
					if (!equal(*(p0 + k), *(p1 + k), sps)){
						changed[i] = true;
						counter++;
						break;
					}
				}
			}
		}

		splineTableSaved = splineTable;
	}

	// Integration over all quadrature points to calculate the Z value for each window
	// here pq.xx contains pre-calcuated summations of x, x*y, x^2*y, ... of the quadrature points
	// plogw contains the log of the quadrature weights (implicitly including the biasing potential through Gauss-Hermite rules) 

	unsigned nxy = nx*ny;

	unsigned counter = 0;
	for (unsigned iq = 0; iq < numberQuadPoints; iq++){
		pcc[iq] = pChanged[pTbl[iq]];
		if (pcc[iq]){
			const double* px = &(pq[iq].xx[0]);
			const double* p = p0 + (pTbl[iq])*inc + shift;
			double f = 0.0;
			for (unsigned i = 0; i < nxy; i++) f += px[i] * p[i];
			ptt[counter++] = pLogw[iq] - f;
		}
	}

	// Weighted function value for each quadrature point
	for (int iq = 0; iq < counter; iq++) ptt[iq] = exp(ptt[iq]);

	// add contribution back to each window (also calculate the contribution of the x average) 
	counter = 0;
	/*
	for (unsigned iq = 0; iq < numberQuadPoints; iq++){
		if (pcc[iq]){
			unsigned wi = pWin[iq];
			double ttt = ptt[counter++];
			double d1 = ttt;
			double d2 = pc_X0[iq] * ttt;

			plogz[wi] += (d1 - pc[iq]);
			psum_x[wi] += (d2 - pc_X[iq]);

			pc[iq] = d1;
			pc_X[iq] = d2;
		}
	}
	*/
	for (unsigned i = 0; i < logZ.size(); i++){
		plogz[i] = 0.0; psum_x[i] = 0.0;
	}
	for (unsigned iq = 0; iq < numberQuadPoints; iq++){
		if (pcc[iq]){
			double ttt = ptt[counter++];
			double d1 = pc_X0[iq] * ttt;
			pc[iq] = ttt;
			pc_X[iq] = d1;
		}
		plogz[pWin[iq]] += pc[iq];
		psum_x[pWin[iq]] += pc_X[iq];
	}

	Zsaved = logZ;

	xAve = 0.0;
	for (unsigned window = 0; window < numberWindow; window++) xAve += psum_x[window];

	double totalZ = 0.0;
	for (unsigned window = 0; window < numberWindow; window++) {
		totalZ += logZ[window];
		logZ[window] = log(logZ[window]);
	}

	xAve /= totalZ;

	return;
}

double MiscHelper::CalculateSplineSummation(
	const unsigned dim,
	const OneDDataSet& splineTable,
	const std::vector<SplineNode>& splineNodes,
	const bool useBaye){

	double sum = 0.0;

	const double* p0 = splineTable.data();
	unsigned nx = 4, ny = dim*dim, shift = 2 * dim, inc = 4 * dim*dim + shift;
	unsigned numberNodes = splineNodes.size();

	// Calculate the function value summation of all data points through spline-lookup
	// assuming that function is stored as the splineTable and 
	// all summations of x^n * y^m terms are pre-calculated and stored in SpineNode objects 
	unsigned counter = 0;
	const SplineNode* p0Node = splineNodes.data();
	for (unsigned node = 0; node <numberNodes; node++){

		const SplineNode* pNode = p0Node + node;
		unsigned index_x = pNode->index[0];
		unsigned index_y = pNode->index[1];
		if ((index_x == 0) || (dim>1 && index_y == 0)) continue;
		unsigned index_f = (counter++)*inc + shift;

		const double* p = p0 + index_f;
		if ((pNode->x[0][0])>1e-5){
			// Use pointer to avoid the expansive [] operator of std::vector

			double tt = 0.0;
			for (unsigned i = 0; i<nx; i++)
				for (unsigned j = 0; j<ny; j++) tt += (p[j + i*ny])*(pNode->x[i][j]);

			sum += tt;
		}
		if (useBaye){ // under experimental tests
			double a = 0.2, c = 10.0;
			sum -= a*(p[0] + c*log(pNode->minDist));
		}
	}

	return sum;

}



void MiscHelper::CalculateProbability(
	const NDDataSet& x,
	const OneDDataSet& u,
	const std::vector<bool>& mask,
	const double ptot,
	const double lambda,
	OneDDataSet& prob,
	double& xAve,
	double& variance,
	double& divergence){

	static OneDDataSet prob0;

	unsigned dataSize = prob.size();
	for (unsigned i = 0; i < dataSize; i++) prob[i] = 0;

	std::vector<double>::const_iterator minU = std::min_element(u.begin(), u.end());

	double ttp = 0.0;
	unsigned counter = 0;

	xAve = 0.0;
	double x2 = 0.0;
	for (unsigned i = 0; i<dataSize; i++)	{
		if (mask[i]){
			counter++;
			prob[i] = exp(-(u[i] - *minU) - lambda*x[0][i]);
			ttp += prob[i];
			xAve += x[0][i] * prob[i];
			x2 += x[0][i] * x[0][i] * prob[i];
		}
	}
	xAve /= ttp;
	x2 /= ttp;
	variance = x2 - xAve*xAve;
	ttp /= ptot;
	for (unsigned i = 0; i<dataSize; i++) prob[i] /= (ttp);

	if (equal(lambda, 0)) {
		prob0 = prob;
		divergence = 0.0;
	}
	else {
		for (unsigned i = 0; i < dataSize; i++) {
			double tt = prob[i] / prob0[i];
			if (tt>0) divergence += prob[i] * log(prob[i] / prob0[i]);
		}
	}

}