// Periodicity.h: interface for the Periodicity class.
//
//
//
//////////////////////////////////////////////////////////////////////
#ifndef _Periodicity_H
#define _Periodicity_H

#include <vector>

namespace {static const double constBufferFactor=0.5+0.00001;};

class CommandLineManager;

class Periodicity
{

public:

	struct PeriodicalCondition {
		bool isPeriodic;
		double min;
		double max;
		double middle;
		double period;
		double bufferFactor;
		PeriodicalCondition(const double fmin, const double fmax, const double fbuffer):
			min(fmin),max(fmax),middle( (fmin+fmax)/2.0), period(fmax-fmin), bufferFactor(fbuffer), isPeriodic(true){
		};
		PeriodicalCondition(): min(-1.0e99),max(1.0e99),middle( 0.0), period(0.0), bufferFactor(constBufferFactor) {
			isPeriodic=false;
		};
	};

	typedef std::vector<PeriodicalCondition> PeriodicalConditionSet ;

	static Periodicity& Instance();
	void Initial(CommandLineManager* manager);

	virtual ~Periodicity();
	const PeriodicalConditionSet& GetConditions() const { return m_conditions; };

private:

	static const unsigned maxDim=4;

	PeriodicalConditionSet m_conditions;
	CommandLineManager* m_manager;

	Periodicity();
};

#endif // // !defined(AFX_Periodicity_H__CB6B9FD7_BF49_4588_9743_36764E05B87F__INCLUDED_)
