#include <vector>
extern "C" {
#include "dcdreader.h"
}

#include "DcdFileReader.h"
#define INIT_NUM_INDICES 4

DcdFileReader::DcdFileReader(const std::string& fileName)
{

	m_dcdFileList.clear();
	m_dcdFileList.resize(1, fileName);

	m_first=1;
	m_last=-1;
	m_stride=1;

}

int DcdFileReader::Read()
{
	return ReadDCD(m_dcdFileList, m_first, m_last, m_stride);
}

DcdFileReader::~DcdFileReader(void)
{
}

int DcdFileReader::ReadDCD(const std::vector<std::string> & dcdFileList, int first, int last, int stride) {

  DCDhandle h_in, h_out;
  int retval;
  float *x, *y, *z;
  int i, iset, nset;
  int just_num = 0;

    
  /* 
   * Open the input dcd files so we can read its header.  Need to know how
   * many timesteps there will be, and it's also a good idea to make sure all
   * the files are present and readable before we start filling up the disk...
   */ 

  iset = 0;
  nset = 0;

  unsigned numberDCDfiles=dcdFileList.size();
  for (unsigned ifile=0; ifile < numberDCDfiles; ifile++) {
    memset(&h_in, 0, sizeof(DCDhandle));

	const char* currentFile=dcdFileList[ifile].c_str();
    retval = dcd_read_open(&h_in, currentFile);
    if (retval) {
      printf("failure opening %s, returned %d.\n",currentFile, retval);
      return 1;
    }
    if (just_num) {
      printf("DCD file %s contains %d frames\n", currentFile, h_in.NSET); 
    }
    nset += h_in.NSET;
    dcd_read_close(&h_in);
  }

  if (last < 0) last = nset;
  if (last < first) {
    printf("Error: last must be greater than or equal to first.\n");
    exit(1);
  }

  printf("Reading frames %d through %d with stride of %d.\n", first, last, stride);

  nset = 1+(last-first)/stride;

  x = new float[h_in.N];
  y = new float[h_in.N];
  z = new float[h_in.N];

  int numfromfile = 0;
  for (unsigned ifile=0; ifile < numberDCDfiles; ifile++) {

    memset(&h_in, 0, sizeof(DCDhandle));
	const char* currentFile=dcdFileList[ifile].c_str();
    retval = dcd_read_open(&h_in, currentFile);
    if (retval) {
      printf("failure opening %s, returned %d.\n",currentFile, retval);
      return 1;
    }
    printf("Reading %d frames from file %s...",h_in.NSET, currentFile);
    fflush(stdout);

    if (iset + h_in.NSET < first) {
      printf("wrote 0 frames.\n");
      iset += h_in.NSET;
    } else {
      for (i=0; i<h_in.NSET; i++) {
        dcd_read_step(&h_in, h_in.ISTART + i*h_in.NSAVC, x,y,z);
		printf("Frame# %d: %3.4f %3.4f %3.4f .\n", i, x[i], y[i], z[i]);
        iset++;
        if (iset < first) {
          continue;
        }
        if (iset > last) {
          break;
        }
        if ((iset-first) % stride) {
          continue;
        }
        numfromfile++;
      }
      dcd_read_close(&h_in);
    }
  }

 

  free(x); 
  free(y); 
  free(z); 
}

