
#include "../../MathTools/include/MathCommon.h"
#include "../../MathTools/include/PhysicalConstants.h"
#include "../include/Output.h"
#include "../include/FileReader.h"
#include "../include/StringParser.h"
#include "../include/FunctionFileHandler.h"

void FunctionFileHandler::WriteToFile(const std::string& key, const NDDataSet& positions, const OneDDataSet& values
	, const bool unintInKT, const double temperature){
	 
	double beta = 1.0;
	if (unintInKT) beta=1.0/(PhysicalConstants::GasConstantInKcalPerMole*temperature);

	std::stringstream ss(std::stringstream::in | std::stringstream::out);
	ss << std::setiosflags(std::ios::fixed);

	unsigned dim=positions.size();
	if (dim==1){
		for (unsigned i=0; i< positions[0].size(); i++){
			ss<< std::setw(20) <<  std::setprecision(8) << positions[0][i];
			ss<< std::setw(20) <<  std::setprecision(8) << values[i]/beta;
			ss<< std::endl;
		}
	}
	if (dim==2){
		unsigned counter=0;
		for (unsigned j=0; j< positions[1].size(); j++){
			for (unsigned i=0; i< positions[0].size(); i++){
				ss<< std::setw(20) <<  std::setprecision(8) << positions[0][i];
				ss<< std::setw(20) <<  std::setprecision(8) << positions[1][j];
				ss<< std::setw(20) <<  std::setprecision(8) << values[counter++]/beta;
				ss<< std::endl;
			}
		}
	}
	Output& output=Output::Instance();
	output.AddValue(key, ss);
}

void FunctionFileHandler::ReadFromFile(const std::string& fileName, NDDataSet& positions, OneDDataSet& values, 
	const bool unintInKT, const double temperature){

	double beta = 1.0;
	if (unintInKT) beta = 1.0 / (PhysicalConstants::GasConstantInKcalPerMole*temperature);
	
	// Read the data file
	FileReader reader(fileName);
	StringParser parser(reader.GetContains(),"\r\n");

	std::vector<std::string> entries=parser.GetTokens();
	unsigned numberPoints=entries.size();
	for (int i=numberPoints-1; i>=0 ; i-- )	if (entries[i][0]=='#') entries.erase(entries.begin()+i);
	numberPoints=entries.size();

	StringParser testParser(entries[0]," \t");
	const std::vector<std::string>& testTokens=testParser.GetTokens();

	unsigned dim=(testTokens.size()-1);
	
	positions.clear();
	values.clear();

	for (unsigned i=0; i< numberPoints; i++ ){
	
		const std::string& entry=entries[i];
		StringParser parser(entry," \t");
		
		const std::vector<std::string>& tokens=parser.GetTokens();
		if (tokens.size()>0) {
			NDData temp(dim, 0.0);
			if ( tokens.size()>=(dim+1) ){
				for (unsigned j=0; j< dim; j++) temp[j]=atof(tokens[j].c_str());
				positions.push_back(temp);
				values.push_back(beta*atof(tokens[dim].c_str()));
			}  // if ( tokens.size()>(numberDimension+1)
		} // if (tokens.size()>0)
	} // for (int i=0;...

	assert(positions.size() == values.size());

	return;
}


