#include <fstream>
#include <iostream>
#include <ctime>
#include <boost/algorithm/string.hpp>


#include "../include/Output.h"
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Output::~Output()
{

}

Output& Output::Instance(){

	static Output thisInstance;
	return thisInstance;

}

Output::Output()
{
	m_keyList.clear();
}

int Output::FindKey(const std::string& key){

	int result=-1;
	for (unsigned i=0; i<m_keyList.size(); i++){
		if (boost::iequals(m_keyList[i].key,key)){
			result=i;
			break;
		}
	}
	return result;
}

void Output::AddValue(const std::string& key, const std::string& value, bool addTimeStamp)
{
	static bool timerOn = false;
	static time_t startTime;

	std::string myValue("");
	if (addTimeStamp){

		if (!timerOn){
			timerOn = true;
			time(&startTime);
		}

		time_t rawtime;
		struct tm * timeinfo;

		time ( &rawtime );
		timeinfo = localtime ( &rawtime );

		myValue = "\nCurrent local time and date: ";
		myValue.append(asctime(timeinfo));

		int diff = rawtime - startTime;
		char buff[100];
		sprintf(buff, "Escaped time: %d seconds. \n", diff);
		myValue.append(buff);

	} 
	myValue.append(value);

	int index=FindKey(key);
	if (index>=0){
		m_keyList[index].value.append(myValue);
	} else {
		m_keyList.push_back(entry(key,myValue));
		index=m_keyList.size()-1;
	}
	PrintOut(key, myValue, false);
}

void Output::AddValue(const std::string& key, const std::stringstream& value, bool addTimeStamp)
{
	AddValue(key, value.str(), addTimeStamp);
}

void Output::SetFileName(const std::string& key, const std::string& filename){

	int index=FindKey(key);
	if (index>=0) m_keyList[index].filename=filename;
	else m_keyList.push_back(entry(key,"",filename));
	
	// clean up the current file if existing
	PrintOut(key, true);
}

void Output::PrintOut(const std::string& key, const std::string& info, const bool overwrite){

	using namespace std;

	int index=FindKey(key);
	if (index>=0) {
		std::string filename=m_keyList[index].filename;
		bool fileOpened=false;
		ofstream myfile;
		std::string extra("");

		try {
			if (filename.size()>0){
				ios_base::openmode mode = ios::out;
				if (!overwrite) mode = mode | ios::app | ios::ate ;
				myfile.open (filename.c_str(), mode);
				fileOpened=myfile.is_open();
			}
		} catch (...) {
			extra.append("\n ** Warning: problem with opening the assigned file ");
			extra.append(filename);
			extra.append(".  The output will be displayed on screen instead. \n");
		}
		if (fileOpened)	{
			myfile << info.c_str();
		} else {
			cout<< extra.c_str() <<info.c_str();
		}
	}
}


void Output::PrintOut(const std::string& key, const bool overwrite){

	int index=FindKey(key);
	if (index>=0) {
		PrintOut(key, m_keyList[index].value, overwrite);
	}
}
