
#include "../../MathTools/include/MathCommon.h"
#include "../../MathTools/include/MathFunctionOnGrid.h"
#include "../include/Output.h"
#include "../include/FileReader.h"
#include "../include/StringParser.h"
#include "../include/FunctionFileHandler.h"
#include "../include/SplineFileHandler.h"

void SplineFileHandler::WriteToFile(const NDDataSet& positions, const OneDDataSet& values, const double temperature){
	
	double min = *(std::min_element(values.begin(), values.end()));
	
	std::string key = "spline output";	
	OneDDataSet shifted = values;
	
	for (unsigned i = 0; i < shifted.size(); i++) shifted[i] -=min;
	FunctionFileHandler::WriteToFile(key, positions, shifted, true, temperature);

}

void SplineFileHandler::ReadFromFile(const std::string& fileName, NDDataSet& inPositions, OneDDataSet& inValues, 
	const double temperature, const bool keepSpline){

	NDDataSet positions, unsortedPos;
	OneDDataSet values;

	FunctionFileHandler::ReadFromFile(fileName, unsortedPos, values, true, temperature);
	
	unsigned numberPoints = unsortedPos.size();
	unsigned dim = unsortedPos[0].size();

	assert(numberPoints>0 && dim > 0);

	double min = *(std::min_element(values.begin(), values.end()));
	for (unsigned i = 0; i < numberPoints; i++)	values[i] -= min;


	unsigned nx = 0, ny = 0;

	if (dim == 1){
		positions.resize(1, NDData(numberPoints, 0));
		for (unsigned i = 0; i< numberPoints; i++) positions[0][i] = unsortedPos[i][0];
	}

	if (dim == 2){
		for (unsigned i = 0; i< numberPoints - 1; i++){
			if (!equal(unsortedPos[i][1], unsortedPos[i + 1][1])){
				nx = i + 1;
				break;
			}
		}
		if (numberPoints%nx != 0) return;

		ny = numberPoints / nx;
		positions.resize(2);
		positions[0].resize(nx, 0.0);
		positions[1].resize(ny, 0.0);

		for (unsigned i = 0; i< numberPoints; i++){
			unsigned iy = (i / nx);
			unsigned ix = i - iy*nx;
			positions[0][ix] = unsortedPos[i][0];
			positions[1][iy] = unsortedPos[i][1];
		}
	}

	if (keepSpline){
		inPositions=positions;
		inValues=values;
	} else {
		MathFunctionOnGrid::ResampleCubicSpline(positions, values, inPositions, inValues);
		inValues = values;
	}

	return;
}
