
#include "MathTools/include/MathCommon.h"
#include "MathTools/include/MathFunctionOnGrid.h"
#include "BEDAMMLEFinder.h"


double BEDAMMLEFinder::FunctionValue (const OneDDataSet& arg) {

	double beta=GetModelFunction()->GetFeature()[0][0]; // 1/RT

	OneDDataSet parameters(arg);
	unsigned numberParameter=parameters.size();

//parameters[0]=0.5;// lamda*
parameters[1]=0.5;// Pa*
parameters[2]=1.0-parameters[1];// Pb*
//parameters[3]=-10.0;// Ua*
//parameters[4]=-20.0;// Ub*
//parameters[5]=2.5;// SigmaA**2
//parameters[6]=2.5;// SigmaB**2

	unsigned numberWindow=GetRawData().size();
	std::vector<double> logL(numberWindow, 0.0);
	double sum=0.0;
	double p0=parameters[0];
	double p5=parameters[5]*parameters[5];
	double p6=parameters[6]*parameters[6];
	double ua,ub,pa,pb,xx;

	for (unsigned window=0; window<numberWindow; window++){

		unsigned sampleSize=GetRawData()[window].size();
		const std::vector<double>& rawData_x=GetRawData()[window][0];

		std::vector<double> tt(sampleSize,0.0), temp1(sampleSize,0.0), temp2(sampleSize,0.0);

		ua=parameters[3]-beta*p5*(m_lambda[window]-p0);
		ub=parameters[4]-beta*p6*(m_lambda[window]-p0);

		pa=-(parameters[3]+ua)*(parameters[3]-ua)/p5/2.0;
		pa=parameters[1]*exp(pa);
		pb=-(parameters[4]+ub)*(parameters[4]-ub)/p6/2.0;
		pb=parameters[2]*exp(pb);

		xx=pa+pb;
		pa/=xx; pb/=xx;

		MathFunctionOnGrid::Gaussian(ua, p5, rawData_x, temp1, true);
		MathFunctionOnGrid::Gaussian(ub, p6, rawData_x, temp2, true);

		for (unsigned i=0; i<sampleSize; i++) {
			xx=pa*temp1[i]+pb*temp2[i];
			if (xx<mini_double) xx=mini_double;
			logL[window]+=log(xx);	
			tt[i]=-30+double(i)*30.0/double(sampleSize);
		}
		sum+=logL[window];

		//TEST
		MathFunctionOnGrid::Gaussian(ua, p5, tt, temp1, true);
		MathFunctionOnGrid::Gaussian(ub, p6, tt, temp2, true);
		for (unsigned i=0; i<sampleSize; i++) {
			temp1[i]=pa*temp1[i]+pb*temp2[i];	
		}

		xx=MathFunctionOnGrid::DefiniteIntegral(tt, temp1);
		//TEST

	}

	return sum/normalFactor;
}

OneDDataSet BEDAMMLEFinder::Derivative(const OneDDataSet& arg) const{

	return arg;

}

void BEDAMMLEFinder::OutputResults(const std::vector<double>& parameter){

}
