#include <math.h>
#include <iostream>
#include <limits>
#include <algorithm>

#include "Utils/include/FileReader.h"
#include "Utils/include/StringParser.h"
#include "MathTools/include/PhysicalConstants.h"
#include "MathTools/include/MathFunctionOnGrid.h"
#include "MathTools/include/ModelFunction.h"

#include "Common.h"
#include "MLEFinder.h"
#include "BEDAMWindow.h"

#include "BEDAMWindowSet.h"

BEDAMWindowSet::BEDAMWindowSet(const std::string& metafileName, const int dataFreq)
{
	GetCurrentWindowList().clear();
	ReadBEDAMFiles(metafileName, dataFreq);
}

BEDAMWindowSet::~BEDAMWindowSet()
{
}

void BEDAMWindowSet::GetWindowSetRange(NDData& minR, NDData& maxR){
	
// TODO--change to N-D

	minR.resize(1,1.0e99);
	maxR.resize(1,-1.0e99);

	std::vector<DataWindow*>& list=GetCurrentWindowList();

	for (unsigned i=0; i< list.size(); i++){

		const double lambda=((BEDAMWindow*) list[i])->GetLambda();

		if (minR[0]>lambda) minR[0]=lambda;
		if (maxR[0]<lambda) maxR[0]=lambda;
	}

	return;


}



void BEDAMWindowSet::ReadBEDAMFiles(const std::string& metafileName, const int dataFreq){

	FileReader reader(metafileName);
	StringParser parser(reader.GetContains(),"\r\n");

	const std::vector<std::string>& entries=parser.GetTokens();

	assert(entries.size()>=2);

	FileReader lambda_reader(entries[0]);
	StringParser lambda_parser(lambda_reader.GetContains(),"\r\n");
	const std::vector<std::string>& lambda_entries=lambda_parser.GetTokens();
	unsigned numberWindows=lambda_entries.size();

	std::vector<BEDAMWindow*> pWindowList(numberWindows);
	for (unsigned i=0; i< numberWindows; i++){
		NDDataSet dataSet;
		dataSet.clear();
		double myLambda=atof(lambda_entries[i].c_str());
		pWindowList[i] = new BEDAMWindow(myLambda, dataSet);
	}

	FileReader data_reader(entries[1]);
	StringParser data_parser(data_reader.GetContains(),"\r\n");
	const std::vector<std::string>& data_lines=data_parser.GetTokens();
	unsigned numberDataLines=data_lines.size();

	unsigned increase=( dataFreq>=1 ?  dataFreq : 1);
	for (unsigned i=0; i< numberDataLines; i+=increase ){
		StringParser line_parser(data_lines[i]," \t");
		const std::vector<std::string>& data_entries=line_parser.GetTokens();
		assert(data_entries.size()==numberWindows);

		for (unsigned j=0; j< numberWindows; j++){
			std::vector<double> temp(1,atof(data_entries[j].c_str()));
			NDDataSet& data=pWindowList[j]->GetRawData();
			data.push_back(temp);
		}
	}

	for (unsigned i=0; i< numberWindows; i++){
		if (pWindowList[i]->TestData())GetCurrentWindowList().push_back(pWindowList[i]);
		else delete pWindowList[i];
	}

}


