#pragma once

#ifndef _DataWindow_H
#define _DataWindow_H

#include "MathTools/include/MathCommon.h"

class WindowSet;
class MLEFinder;

class DataWindow
{

	friend class WindowSet;
	friend class MLEFinder;

protected:  

	unsigned GetDimensionality(){ return m_dimensionality; }
	void SetDimensionality(unsigned dim){ m_dimensionality=dim; }

	virtual bool TestData()=0;
	virtual void DoStat()=0;
	virtual void PrintDataInfo()=0;

	DataWindow(){ InitTemperture(); };
	DataWindow(const NDDataSet& rawData){ InitTemperture(); m_positions = rawData; };

	virtual ~DataWindow(){};

	virtual NDDataSet& GetRawData() { return m_positions; };
	virtual void CleanRawData() { m_positions.clear(); };

	const DataRangeSet& GetDataRange() const { return m_dataRange; };
	void SetDataRange(const DataRangeSet& dataRange){ m_dataRange=dataRange; };

	const NDData& GetIntergationSize() const { return m_intergationSize; };
	void SetIntegrationSize(const NDData& intSize){ m_intergationSize=intSize; };

	double GetTemperature() const{ return m_temp; };
	void SetTemperature(const double temp){ m_temp=temp; };

	double GetWindowWeight() const{ return m_windowWeight; };
	void SetWindowWeight(const double weight){ m_windowWeight=weight; };	

	virtual const OneDDataSet& GetDataWeight() const{ return m_dataWeight; };
	virtual void SetDataWeight(){ };

private:
	
	NDDataSet m_positions;
	OneDDataSet m_dataWeight;

	DataRangeSet m_dataRange;
	unsigned m_dimensionality;

	NDData m_intergationSize;

	double m_temp;
	double m_windowWeight;

	void InitTemperture();
};

#endif // _DataWindow_H
