#pragma once

#ifndef _MLEFinder_H
#define _MLEFinder_H

#include "Common.h"

#include "MathTools/include/MathCommon.h"
#include "MathTools/include/ModelFunction.h"
#include "MathTools/include/Maximizer.h"

class MathFunctionOnGrid;
class WindowSet;

class MLEFinder : public Maximizer
{
public:

	MLEFinder(WindowSet* pSet, bool derivative);
	~MLEFinder(void){};

	void FindML(std::vector<double>& parametersInVector);
	void FindML(std::vector<double>& parametersInVector, std::vector<double>& multipliers);

	const std::auto_ptr<ModelFunction>& GetModelFunction() const { return m_pFunction; };

	void (* GetFunctionPointer()) 
		(const NDDataSet& feature,
		const std::vector<double>& parameter,
		const NDDataSet& x, 
		OneDDataSet& y); 

protected:

	WindowSet* GetWindowSet(){ return m_pSet;} ;
	const NDDataSetCollection& GetRawData() const { return m_rawData; };
	const NDData& GetWeight() const;
	const double GetTemperature() const;
	virtual void OutputResults(const std::vector<double>& parameter)=0;

private:

	WindowSet* m_pSet;
	NDDataSetCollection m_rawData;

	std::auto_ptr<ModelFunction> m_pFunction;

private:

	MLEFinder(void);
	MLEFinder(const MLEFinder&);

};

#endif // _MLEFinder_H
