#pragma once

#ifndef _SetDecorator_H
#define _SetDecorator_H

#include <boost/shared_ptr.hpp>
#include "WindowDataType.h"
#include "WindowSet.h"

class SetDecorator
{

public:

	SetDecorator(boost::shared_ptr<WindowSet> pSet);
	virtual ~SetDecorator(void);

	virtual void SetupOptions()=0;
	virtual void PerformAnalysis()=0;
	virtual void OutputResults();

protected:

	boost::shared_ptr<WindowSet> GetWindowSet(){ return m_pSet; };
	
	NDDataSet& GetX(){ return m_x; };		
	OneDDataSet& GetU(){ return m_u; };	
	std::vector<bool>& GetMask(){ return m_mask; };

	NDDataSet& GetFeature(){ return m_feature; };		
	OneDDataSet& GetParameters(){ return m_parameters; };	

	virtual void AddJacobian();
	virtual void OutputFreeEnergyAndDensity();
	virtual void OutputPath();

private:
	
	boost::shared_ptr<WindowSet> m_pSet;
	NDDataSet m_x;
	double m_shift;

	OneDDataSet m_u;

	std::vector<bool> m_mask;

	NDDataSet m_feature;
	OneDDataSet m_parameters;
};

#endif // _SetDecorator_H
