#pragma once

#ifndef _USMLEFinder_H
#define _USMLEFinder_H

#include "Common.h"
#include "MathTools/include/ModelFunction.h"
#include "Prior.h"
#include "USWindowSet.h"
#include "MLEFinder.h"

class MathFunctionOnGrid;


class USMLEFinder: public MLEFinder
{
public:

	USMLEFinder(WindowSet* pSet);
	~USMLEFinder(void) {};

	virtual double FunctionValue(const OneDDataSet& arg);
	virtual OneDDataSet Derivative(const OneDDataSet& arg) const;
	virtual void OutputResults(const std::vector<double>& parameter);

	void SetPrior(const NDDataSet& positions, const OneDDataSet& values, const Prior::PriorType type, const double weight=1.0);
	void SetPrior(const double weight=0.001);

	void CopyBoundaryParameters(std::vector<double>& parameters);

private:

	void BuildQuadraturePointList(const bool dataWeight, const double& logWeightCut=-15.0);

	const std::vector<SplineNode>& m_splineNodes;
	const NDDataSet& m_nodePositions;
	const std::vector<QuadratureRuleSet>& m_ruleSets;
	const std::vector<int>& m_periodicCopyList;

	std::vector<double> m_logZ;
	std::vector<QPoint> m_quadraturePointList;

private:

	std::auto_ptr<Prior> m_pPrior;
	
	USMLEFinder(void);
	USMLEFinder(const USMLEFinder&);
};


#endif // _USMLEFinder_H
