#pragma once

#ifndef _USWindow_H
#define _USWindow_H

#include "MathTools/include/MathCommon.h"
#include "DataWindow.h"

class USWindowSet; 
class USSetDecorator;

class USWindow: public DataWindow
{

	friend class USWindowSet;
	friend class USSetDecorator;

protected:

	virtual ~USWindow(void);
	USWindow(const std::string& fileName, const NDData& eqPosition, const NDData& forceConst, const double weight, const int dataFreq);
	USWindow(const NDDataSet& rawData, const NDData& eqPosition, const NDData& forceConst, const double weight, bool symmetricPair=false);

	static std::vector<USWindow*> SplitWindow(USWindow* parentWindow, unsigned numberSplitPerDimension); 

	virtual bool TestData();
	virtual void DoStat();
	virtual void PrintDataInfo();

	const GuassianDistributionSet& GetBiasing() const { return m_biasing; };
	const GuassianDistributionSet& GetObserved() const { return m_observed; };
	const GuassianDistributionSet& GetBestGuess() const { return m_bestGuess; };
	const QuadratureRuleSet& GetQuadratureRuleSet() { return m_ruleSet; };

	const NDData& GetEqPosition() const { return m_eqPosition; };
	const NDData& GetForceConst() const { return m_forceConst; };

	static double defaultSpacing[2]; 

private:

	USWindow();

	// Internal routines

	void ReadDataFile(const std::string& fileName, const int dataFreq);
	void ExtractBiasingInfo(const NDData& biasingEqposition, const NDData& forceConst);

	void BuildUpQuadratureRules();

	GuassianDistributionSet m_biasing, m_observed, m_bestGuess;
	NDData m_eqPosition, m_forceConst;

	QuadratureRuleSet m_ruleSet;
};

#endif //
