#pragma once
#ifndef _USWindowSet_H
#define _USWindowSet_H

#include "MathTools/include/MathCommon.h"
#include "WindowSet.h"

class USWindow; 
class USMLEFinder;
class USSetDecorator;

class USWindowSet : public WindowSet
{

	friend class USMLEFinder;
	friend class USSetDecorator;

public:

	USWindowSet(const std::string& metafileName, const  int dataFreq);
	~USWindowSet(void);

	virtual void GetWindowSetRange(NDData& minR, NDData& maxR);
	const std::vector<SplineNode>& GetSplineNodes() const { return m_splineNodes; };
	const NDDataSet& GetNodePositions() const { return m_nodePositions; };
	const std::vector<int>& GetPeriodicCopyList() const { return m_periodicCopyList; };

	const std::vector<QuadratureRuleSet>& GetQuadratureRuleSets() const { return m_ruleSets; };

public:

	static void BuildUpQuadratureRules(const DataRangeSet& dataRange, const GuassianDistributionSet& biasing, QuadratureRuleSet& ruleSet, NDData& intSize);

protected:

	void ResetSplineNodes(const NDDataSet& newNodes);

private:

	bool AddWindow(USWindow* pWindow, std::string type);

	// Internal routines
	void ReadUSFiles(const std::string& metafileName, const int dataFreq);
	void AddImageWindows();
	void CalculateSplineRange();

	void ResetSplineNodes();
	void BuildUpSplineNodes();
	void BuildUpQuadratureRules();
	void BuildPeriodicCopy();

private:

	std::vector<SplineNode> m_splineNodes;
	std::vector<QuadratureRuleSet> m_ruleSets;
	NDDataSet m_nodePositions; 
	
	std::vector<int> m_periodicCopyList;

	USWindowSet();
	USWindowSet(const USWindowSet&);

};

#endif //
