#include <assert.h>

#include "Utils/include/Periodicity.h"

#include "MathTools/include/MathCommon.h"
#include "MathTools/include/ModelFunction.h"
#include "MathTools/include/PeriodicTools.h"

#include "DataWindow.h"
#include "WindowSet.h"

WindowSet::WindowSet(){

	m_modelFunctionType=ModelFunction::typeOneD_CubicSpline;
	m_currentWindowList.clear();
}

WindowSet::~WindowSet(){

	for (unsigned i=0; i<  m_currentWindowList.size(); i++ ){
		if (m_currentWindowList[i]) delete m_currentWindowList[i];
	}
}

void WindowSet::SetUpDimensionality(){

	m_dimensionality=999;
	for (unsigned i=0; i<  m_currentWindowList.size(); i++ ){
		unsigned dim=m_currentWindowList[i]->GetDimensionality();
		// Use the minimal dim amoung all windows as the dim the window set
		if ( dim < m_dimensionality) m_dimensionality=dim;
	}
}

void WindowSet::SetWindowWeight(){

	m_windowWeight.clear();
	m_windowWeight.resize(m_currentWindowList.size(), 1.0);
	for (unsigned i=0; i<  m_currentWindowList.size(); i++ ){
		m_windowWeight[i] = m_currentWindowList[i]->GetWindowWeight();
	}
}

void WindowSet::SetModelFunctionType(ModelFunction::functionType type){
	m_modelFunctionType=type;
}

const ModelFunction::functionType WindowSet::GetModelFunctionType() const{
	return m_modelFunctionType;
}

void WindowSet::GetDataRange(NDData& minR, NDData& maxR){

	unsigned dim=GetDimensionality();
	assert(dim>=1);

	minR.resize(dim,1.0e99);
	maxR.resize(dim,-1.0e99);

	const std::vector<DataWindow*>& list=GetCurrentWindowList();
	const Periodicity::PeriodicalConditionSet& c=Periodicity::Instance().GetConditions();

	for (unsigned window=0; window<list.size(); window++){

		DataWindow* tt=list[window];
		const DataRangeSet& dataRange=tt->GetDataRange();
		for (unsigned i=0; i< dim; i++){
			if (minR[i]>dataRange[i].min) minR[i]=dataRange[i].min;
			if (maxR[i]<dataRange[i].max) maxR[i]=dataRange[i].max;
		}
	}
	for (unsigned i=0; i< dim; i++){
		if (c[i].isPeriodic) {
			minR[i]=std::min(minR[i], c[i].min);
			maxR[i]=std::max(maxR[i], c[i].max);
		}
	}

	return;

}


void WindowSet::GetIntegrationSize(NDData& minInt, NDData& maxInt){

	unsigned dim=GetDimensionality();
	assert(dim>=1);

	minInt.resize(dim,1.0e99);
	maxInt.resize(dim,-1.0e99);

	const std::vector<DataWindow*>& list=GetCurrentWindowList();
	
	for (unsigned window=0; window<list.size(); window++){
		DataWindow* tt=list[window];
		const NDData& intSize=tt->GetIntergationSize();
		for (unsigned i=0; i< dim; i++){
			if (minInt[i]>intSize[i]) minInt[i]=intSize[i];
			if (maxInt[i]<intSize[i]) maxInt[i]=intSize[i];
		}
	}

	return;

}
