#pragma once
#ifndef _WindowSet_H
#define _WindowSet_H

#include <vector>

#include "MathTools/include/MathCommon.h"
#include "MathTools/include/ModelFunction.h"

class DataWindow;
class SetDecorator;

class WindowSet
{
public:

	std::vector<DataWindow*>& GetCurrentWindowList () { return m_currentWindowList; };
	virtual void GetWindowSetRange(NDData& minR, NDData& maxR)=0;

	void GetDataRange(NDData& minR, NDData& maxR);
	void GetIntegrationSize(NDData& minInt, NDData& maxInt);

	void SetModelFunctionType(ModelFunction::functionType type);
	const ModelFunction::functionType GetModelFunctionType() const;

	const NDData& GetWindowWeight() const { return m_windowWeight; };
	unsigned GetDimensionality() const{ return m_dimensionality; };

	double GetTemperature() const{ return m_temp; };
	void SetTemperature(const double temp){ m_temp=temp; };

	~WindowSet();

protected:

	WindowSet();
	void SetUpDimensionality();
	void SetWindowWeight();

private:

	WindowSet(const WindowSet&);

	ModelFunction::functionType m_modelFunctionType;
	std::vector<DataWindow*> m_currentWindowList;
	unsigned m_dimensionality;
	NDData m_windowWeight;
	double m_temp;

public:

	template <class myType> static std::vector<myType*> ChangeType(std::vector<DataWindow*> list){
		std::vector<myType*> changed;
		changed.clear();
		for (unsigned i=0; i<list.size(); i++){
			changed.push_back( (myType*) list[i]);
		}
		return changed;
	}

};


#endif //
