// vFEP.cpp : Defines the entry point for the console application.
//
#include <signal.h>
#include <stdexcept>

#include "Utils/include/Output.h"
#include "Utils/include/Periodicity.h"
#include "Utils/include/SignalHandler.h"
#include "MathTools/include/MathFunctionOnGrid.h"
#include "MathTools/include/ModelFunction.h"
#include "MathTools/include/MathCommon.h"

#include "vFEPCommandManager.h"
#include "Common.h"
#include "WindowDataType.h"
#include "WindowSet.h"
#include "WindowSetFactory.h"
#include "SetDecorator.h"
#include "SetDecoratorFactory.h"


void signal_handler(int sig)
{
	SignalHandler& h=SignalHandler::Instance();
	h.SetSignal(sig, true);
}

int main(int argc, char* argv[])
{

	signal(SIGINT, signal_handler);

	vFEPCommandManager& manager= vFEPCommandManager::Instance(argc, argv);
	Output& output = Output::Instance();

	// Help screen
	if (manager.GetKey("h")||manager.GetKey("hx")){
		manager.PrintOutHelpMessage(manager.GetKey("hx"));
		return 0;
	}

	Periodicity::Instance().Initial(&manager);

// Types of data window  
	WindowDataType::Type type=WindowDataType::UmbrellaSampling;
	std::string fileName="metafile";

	if (manager.GetFileName("m",fileName)) {
		type=WindowDataType::UmbrellaSampling;
	} else { 
		if (manager.GetFileName("b",fileName)) type=WindowDataType::BEDAM;
	}
	if (type==WindowDataType::Unknown){
		std::cout << "Please assign the metafile name and type.  Use -h to see options" << std::endl;
		return 1;
	}

	std::string statfileName("");
	if (manager.GetFileName("stat", statfileName))
		output.SetFileName("Statistics", statfileName);
	
	std::string logfileName("");
	if (manager.GetFileName("log", logfileName)){
		output.SetFileName("program log", logfileName);	
		output.SetFileName("error", logfileName);	
		output.SetFileName("warning", logfileName);	
		output.AddValue("program log", "Starting the vFEP program\n", true);
	}

	try{
		//  Create a set of Windows from the data files
		double temperature = 300;
		if (manager.GetKey("temp")) manager.GetValue("temp", temperature);

		boost::shared_ptr<WindowSet> pCurrentSet = WindowSetFactory::Instance().CreateWindowSet(fileName, type, temperature);

		//  Create a decorator for the window set and deligate the real work to the decorator
		boost::shared_ptr<SetDecorator> pDecorator=SetDecoratorFactory::Instance().CreateSetDecorator(pCurrentSet, type);
	
		pDecorator->SetupOptions();
		pDecorator->PerformAnalysis();
		pDecorator->OutputResults();

	}catch(std::runtime_error& e){
		output.AddValue("error", std::string(e.what()), true);
	}catch(...){
		output.AddValue("error", "Unknown runtime error");
	}

	output.AddValue("program log", "End of the vFEP program\n", true);
	
 	return 0;

}



