#include "vFEPCommandManager.h"

vFEPCommandManager::vFEPCommandManager(void)
{
}

vFEPCommandManager::~vFEPCommandManager(void)
{
}

vFEPCommandManager& vFEPCommandManager::Instance(){

	static vFEPCommandManager thisInstance;
	return thisInstance;

}

vFEPCommandManager& vFEPCommandManager::Instance(int numberArg, char* arg[]){

	vFEPCommandManager& instance=vFEPCommandManager::Instance();
	instance.Initialize(numberArg, arg);
	return instance;
}


void vFEPCommandManager::CreateHelpMessage(){

	std::string message;

	AddHelpMessage("\nAvailable options\n", "");

	message="help (this output) ";
	AddHelpMessage("-h", message);
	
	AddHelpMessage("\nFile options", "");

	message="the metafile for umbrella sampling (also turn on umbrella sampling type calculation; default: metafile) ";
	AddHelpMessage("-m filename", message);
	
	message="the metafile for BEDAM (also turn on BEDAM type calculation) ";
	AddHelpMessage("-b filename", message);
	
	message="Output the free energy profile to file.";
	AddHelpMessage("-fep filename", message );

	message="Output the probability density to file.";
	AddHelpMessage("-prob filename", message );
	
	message="Output the program progress to file.";
	AddHelpMessage("-log filename", message);
	
	message="Output the relative free energy shifts between windows.";
	AddHelpMessage("-Z fileName", message);

	AddHelpMessage("\nData options", "");

	message = "Specify the temperture.";
	AddHelpMessage("-temp double", message);

	message="must be an integer:";
	message.append("\n\t\t dataFrequence=0,1,-1: use all data points (default).");
	message.append("\n\t\t dataFrequence>1: only pickup data points every dataFrequence, e.g., dataFrequence=10 will cause that only the 1st, 11th, 21th , 31th... data points will be used.");
	message.append("\n\t\t dataFrequence<-1: only 1/(-dataFrequence) portion of data points will be used (randomly pickup), e.g., if dataFrequence=-10, 1/10 of total data points will be randomely picked up and used.");
	AddHelpMessage("-s integer", message);

	message="Specify the reference point of x (and y for 2D) where the output function value will be adjuected to zero.";
	AddHelpMessage("-x0 double, -y0 double", message);
	
	message="Specify the scalar factor(s) for the coordinate(s) for the 1st dimension (s1) and the second dimension (s2).  Default: 1.0.";
	AddHelpMessage("-s1 double, -s2 double", message);

	message="The output free energy shift values will be relatived to the first window.";
	AddHelpMessage("-Z0", message);

	message="Assume the biasing force constant defined as k(x-x0)^2, rather than the default way (k/2)(x-x0)^2.";
	AddHelpMessage("-k2", message);

	message="Add the polar coordinate Jocabian correction to the first dimension.";
	AddHelpMessage("-jr1", message);

	message="Add the polar coordinate Jocabian correction to the second dimension.";
	AddHelpMessage("-jr2", message);

	message="Create the missing symmetrical windows in a 2D simulation.";
	AddHelpMessage("-sym ", message);

	message="Must be an integer: Split each window into nsplit (per dimension) child widows.  Only applicable to windows with zero biasing potentials.";
	AddHelpMessage("-nsplit integer", message);

	message="Turn off the automatic addition of periodic boundary window pairs.";
	AddHelpMessage("-no_pb", message);

	message = "Ignor windows with non-consistent data.";
	AddHelpMessage("-del_bad", message);

	message="Turn on the automatic addition of periodic boundary window pairs. (current default)";
	AddHelpMessage("-add_pb", message);

	message="Only use the first n data points for each window.";
	AddHelpMessage("-head integer", message);

	message="Only use the last n data points for each window.";
	AddHelpMessage("-tail integer", message);

	message="Turn on the weight option.  Each window has different weight, specified at the last column of the metafile.";
	AddHelpMessage("-w", message);
	
	AddHelpMessage("\nNumerical options", "");

	message="Number of grid points (-grid for all dimension, -grid1 for the first dimension only, and -grid2 the second dimension only) of the free energy profile.";
	message.append("\n\t\t default value is 200. ");
	AddHelpMessage("-grid (-grid1, -grid2) integer", message);

	message="Add blank lines in the free energy profile output for gnuplot usage";
	AddHelpMessage("-gnu", message);
	
	message="Method to be used to analyze umbrella sampling data:";
	message.append("\n\t\t MLE: use maximum likelihoond method, i.e., vFEP. (default, 1D and 2D).");
	message.append("\n\t\t Derivative, UI: use umbrella integration. (1D only)");
	message.append("\n\t\t WHAM, Wham: use WHAM. (1D only)");
	AddHelpMessage("-t method", message);
	
	message="Interpolation type used in vFEP:";
	message.append("\n\t\t cubic, spline, c, s: use cubic spline (default, 1D and 2D).");
	message.append("\n\t\t rational, r: use rational interpolation. (1D only)");
	AddHelpMessage("-i interpolation:", message);
	
	message="Number of spline nodes used (n1 for x direction and n2 for y direction; Default: automatically determined by the program.).";
	AddHelpMessage("-n1 integer and/or -n2 integer", message);

	message="Turn off the initial guess based on number data points associated with each spline node.";
	message.append("\n\t\t Default is on. ");
	AddHelpMessage("-np", message);

	message="Number of quadrature points used for numerical integrations (default: 48 for Gauss-Hermite quadrature and 12 for Gauss-Legendre quadrature).";
	AddHelpMessage("-nq integer", message);
	message="Use Gauss-Legendre quadrature rules (defauls: use Gauss-Hermite quadrature rules with non-zero biasing potential, otherwise Gauss-Legendre.).";
	AddHelpMessage("-qgl", message);

	AddHelpMessage("\nPath Analysis", "");

	message="Output the path (zero-gradient) points to file. (2D only)";
	AddHelpMessage("-path filename", message );
	
	message="Output the stationary points to file.";
	AddHelpMessage("-st filename", message );
	
	message="Resolutions for the paths and the stationary points for the first (pd1) and the second dimensions (pd2).  Default: 1.0/(number grids).";
	AddHelpMessage("-pd1 double, -pd2 double", message );
	
	message="Include the maximum points (i.e., the points with 2nd derivatives<0) when output stationary points. Default: only minimum points.";
	AddHelpMessage("-max", message );

	AddHelpMessage("\nOther conditions", "");

	message="Define the range of output free energy profile:";
	message.append("\n\t\t -x defines the minimal value and -X the maximal value of the first dimension,");
	message.append("\n\t\t (2D case) -y the minimal value and -Y the maximal value of the second dimension.");
	message.append("\n\t\t Note: the range is only for the free energy profile output, it does not have any effect on the analysis.");
	message.append("\n\t\t Default values: The range(s) will be the input data range(s).");
	AddHelpMessage("-x double (as well as -X, -y, -Y)", message);

	message="Define the range where the data will used:";
	message.append("\n\t\t -dx defines the minimal value and -dX the maximal value of the first dimension,");
	message.append("\n\t\t (2D case) -dy the minimal value and -dY the maximal value of the second dimension.");
	message.append("\n\t\t Any data point outside the defined range will be ignored in the analysis.");
	message.append("\n\t\t Default values: The range(s) will be the input data range(s).");
	AddHelpMessage("-dx double (as well as -dX, -dy, -dY)", message);

	message="turn on periodical boundary conditions:";
	message.append("\n\t\t requires setting min and max x by -x xmin and -X xmax");
	message.append("\n\t\t and (2D case) min and max y by -y ymin and -Y ymax,");
	message.append("\n\t\t where xmin (and ymin) and xmax (and ymax) should correspond to the two ends of the period.");
	message.append("\n\t\t -r turns on all dimensions while -r1 only the 1st dimension -r2 only the 2nd dimension.");
	AddHelpMessage("-r, -r1, -r2", message);

	message="must be a positive real number: the image buffer size factor of the perioduc condition for the first dimension. Default=0.25 for 1D cases and 0.1 for 2D cases";
	AddHelpMessage("-rb1 double", message);

	message="must be a positive real number: the image buffer size factor of the perioduc condition for the second dimension. Default=0.1";
	AddHelpMessage("-rb2 double", message);

	message="must be a positive integer: Maximum number of iterations in optimzation. (default: no limit)";
	AddHelpMessage("-n integer", message);

	message="must be a positive real number: The tolerance (of likelihood in unit of RT) to stop optimzation. (default: 1e-5)";
	AddHelpMessage("-tol double", message);

	AddHelpMessage("\nm FEP options:", "");

	message = "The filename of the initial input spline nodes and node values (spline prior)";
	AddHelpMessage("-spi filename", message);

	message = "Use the input spline node positions.";
	AddHelpMessage("-spk", message);

	message = "The filename of the output spline nodes and node values.";
	AddHelpMessage("-spo filename", message);

	message = "The weight of the initial input free energy profile.";
	AddHelpMessage("-spw double", message);

	message = "The filename of the discrete-point prior.";
	AddHelpMessage("-dpi filename", message);

	message = "The weight of the discrete-point prior.";
	AddHelpMessage("-dpw double", message);

	//
	AddHelpMessageX("\nAdditional hidden (experimental) options", "");

	message="Output more details.";
	AddHelpMessageX("-v", message);

	message="Output statistics info for windows.";
	AddHelpMessageX("-stat fileName", message);

	message = "Turn on the data weight option.";
	message.append("\n\t\t -dw requires an entra numeric entry for each data point as the weight.");
	message.append("\n\t\t -dwe acts similar to -dw, except the input data is interpreted as the energy factor, i.e., the weight will be exp(beta*data_entry.");
	message.append("\n\t\t -dwb uses the biasing potential to calcuate the weight.");
	AddHelpMessageX("-dw, -dwe, -dwb", message);

	message="Number of segments for each window when doing integration with Gauss-Legendre quadrature rules. (default: 8)";
	AddHelpMessageX("-nseg", message);

	message="Turn on the direct FEP calculation.";
	AddHelpMessageX("-dfep", message );

	message = "Turn on the experiment-fitting FEP calculation.";
	AddHelpMessageX("-efep double", message);

	message = "Turn on the experiment-fitting FEP calculation--Lagrange way.";
	AddHelpMessageX("-lagrange double", message);

	message = "Turn on the fourier FEP calculation.";
	AddHelpMessageX("-ffep", message);

	message="Turn on the Bayesian gauess of free energy profile.";
	AddHelpMessageX("-baye", message );

	message="The uniform background prior will be used.";
	AddHelpMessageX("-spb ", message );
	
	message = "Specify the temperture of the prior will be used.";
	AddHelpMessageX("-spt doube", message);

	message="The cut-off for keeping old spline values.";
	AddHelpMessageX("-sps double, defaul 1e-12", message );

	message="The log of cut-off for keeping quadrature points.";
	AddHelpMessageX("-lwc double, defaul -15", message );

	message = "Add padding to the data.";
	AddHelpMessageX("-pad pad_factor", message);

	message = "Ouput the best lambda for experiment-guided simulations";
	AddHelpMessageX("-KL", message);

	message = "Using window average";
	AddHelpMessageX("-wa integer", message);

	message = "Using normal distribution average";
	AddHelpMessageX("-wb integer", message);
}
